#!/bin/sh
# the next line restarts using tclsh \
	exec tclsh "$0" "$@"

set auto_path [linsert $auto_path 0 [file dirname [info script]]]
package require multiplexer 0.2

# commands take the form

# {nick $nick} {cmd $cmd} {msg $msg}

proc cleanhtml { data } {
    return [string map {< &lt; > &gt; & &amp;} $data]
}

proc AccessFilter { chan clientaddress clientport } {
    if { $clientaddress != "127.0.0.1" } {
	puts $chan "Access Denied"
	return -1
    }
}

proc alladd { msg } {
    global connected said
    foreach nm [array names connected] {
	lappend said($nm) $msg
    }
}

proc disconnect { nick } {
    global connected
    alladd "$nick has quit"
    unset connected($nick)
}

# this is the workhorse...

proc Filter { data chan client clientport } {
    global connected saidmax timeout said

    array set line [cleanhtml $data]

    if { $data == "" || ![info exists line(nick)] } {
	return ""
    }
    set nick $line(nick)
    if { ! [info exists said($nick)] } {
	set said($nick) ""
    }

    # so that we tack stuff onto the end
    if { [llength $said($nick)] >= $saidmax } {
	set said($nick) [lrange $said($nick) 1 end]
    }

    # do timeout and log people out
    set tm [clock seconds]
    set connected($nick) $tm
    set limit [expr {$tm - $timeout}]
    foreach cn [array names connected] {
	if { $connected($cn) < $limit } {
	    disconnect $cn
	}
    }

    # the protocol
    switch -exact $line(cmd) {
	# return the buffer
	get {
	    puts $chan $said($nick)
	    return ""
	}
	# exit the user
	quit {
	    disconnect $nick
	    return ""
	}
	# the user joins
	join {
	    alladd "$nick has joined"
	    return ""
	}
	# the user sends us something, tack it onto 'said'
	msg {
	    alladd "$nick says: $line(msg)"
	    return ""
	}
	privmsg {
	    if { [info exists said($line(target))] } {
		lappend said($nick) "you say to $line(target) privately: $line(msg)"
		lappend said($line(target)) "$nick says to you privately: $line(msg)"
	    } else {
		lappend said($nick) "The user $line(target) does not exist"
	    }
	    return ""
	}
	who {
	    puts $chan [array names connected]
	    return ""
	}
	kick {
	    disconnect $line(target)
	    return ""
	}
    }
    return "ERROR, data is: $data"
}

set banlist [list]
array set said {}
array set connected {}
array set privmsgs {}

source serverconfig.tcl

set mp [multiplexer::create]
${mp}::AddAccessFilter AccessFilter
${mp}::AddFilter Filter

${mp}::Config debug 2
${mp}::Config sendtoorigin 0
${mp}::Init $PORT

vwait forever